package bg.bas.dcl.CorpusTools;

import java.io.IOException;
import java.net.MalformedURLException;

import com.gargoylesoftware.htmlunit.FailingHttpStatusCodeException;
import com.gargoylesoftware.htmlunit.WebClient;
import com.gargoylesoftware.htmlunit.html.HtmlForm;
import com.gargoylesoftware.htmlunit.html.HtmlPage;
import com.gargoylesoftware.htmlunit.html.HtmlSubmitInput;
import com.gargoylesoftware.htmlunit.html.HtmlTextInput;

public class AssociationEvaluator {
	
	public int N = 13000000;
	public int N_BNK = 480000000;

	
	public double getChiSquare(int w1freq, int w2freq, int mwefreq, boolean fromBNK) {
		
		int N = (fromBNK)? this.N_BNK : this.N;
		double result = 0;
		
		int[] o = getObserved(w1freq, w2freq, mwefreq, N);
		double[] e = getExpected(o,N);

		for (int i = 0; i< 4; i++) {
			result += Math.pow((o[i]-e[i]),2) / e[i] ;
		}
		
		return result;
	}
	
	public double getLogLikelihood(int w1freq, int w2freq, int mwefreq, boolean fromBNK) {
		
		int N = (fromBNK)? this.N_BNK : this.N;
		double result = 0;
		
		int[] o = getObserved(w1freq, w2freq, mwefreq, N);
		double[] e = getExpected(o,N);

		for (int i = 0; i< 4; i++) {
			result += o[i] * Math.log10(o[i]/e[i]);
		}
		
		return 2*result;
	}

	public double getMutualInformation(int w1freq, int w2freq, int mwefreq, boolean fromBNK) {
		
		int N = (fromBNK)? this.N_BNK : this.N;
		double result = 0;
	
		result = Math.log(((double) N * mwefreq) / ((double) w1freq * w2freq)) / Math.log(2);
		
//		int[] o = getObserved(w1freq, w2freq, mwefreq, N);
//		double[] e = getExpected(o,N);

//		for (int i = 0; i< 4; i++) {
//			result += o[i] * Math.log(o[i]/e[i]) / Math.log(2);
//		}
		
		return result;
	}
	
	public double getEnhancedMutualInformation(int w1freq, int w2freq, int mwefreq, boolean fromBNK) {
		
		int N = (fromBNK)? this.N_BNK : this.N;
		double result = 0;
	
		result = Math.log(
				    ( 1.0*mwefreq*N )
				    / 
					( (w1freq-mwefreq+0.01)*(w2freq-mwefreq+0.01) )
				) / Math.log(2);
		
//		int[] o = getObserved(w1freq, w2freq, mwefreq, N);
//		double[] e = getExpected(o,N);

//		for (int i = 0; i< 4; i++) {
//			result += o[i] * Math.log(o[i]/e[i]) / Math.log(2);
//		}
		
		return result;
	}


	public int[] getObserved(int w1freq, int w2freq, int mwefreq, int N){
		int[] o = new int[4];
		o[0] = mwefreq;
		o[1] = w1freq - mwefreq;
		o[2] = w2freq - mwefreq;
		o[3] = N - w1freq - w2freq + mwefreq;
		
		return o;
	}
	
	public double[] getExpected(int[] o, int N) {
		double[] e = new double[4];
		e[0] = (double)(o[0]+o[1])*(double)(o[0]+o[2]) / (double)N;
		e[1] = (double)(o[0]+o[1])*(double)(o[1]+o[3]) / (double)N;
		e[2] = (double)(o[2]+o[3])*(double)(o[0]+o[2]) / (double)N;
		e[3] = (double)(o[2]+o[3])*(double)(o[1]+o[3]) / (double)N;

		return e;
	}
	
	public int[] getFrequenciesFromBNK(String w1, String w2) {
		
		int[] result = new int[3];
		
//    	String formURL = "http://search.dcl.bas.bg/";

    	
    	for (int k=0; k < 3; k++) {
    		
    		String text = "";
    		if (k == 0) text=w1+"/F/";
    		if (k == 1) text=w2+"/F/";
    		if (k == 2) text="<" +w1+"/F/[0,1]"+w2+"/F/>";
    		
    	    final WebClient webClient = new WebClient();

    	    // Get the first page
    	    HtmlPage page1;
			try {
				page1 = webClient.getPage("http://search.dcl.bas.bg/");

//				System.out.println(page1.asText());
				
	    	    // Get the form that we are dealing with and within that form, 
	    	    // find the submit button and the field that we want to change.
	    	    final HtmlForm form = page1.getForms().get(0);
	
	    	    final HtmlSubmitInput button = form.getInputByValue("Search");
	    	    final HtmlTextInput textField = form.getInputByName("query");
	
	    	    // Change the value of the text field
	    	    textField.setValueAttribute(text);
	
	    	    // Now submit the form by clicking the button and get back the second page.
	    	    final HtmlPage page2 = button.click();
	    	    
	    	    String re = page2.asText();
	    	    int i1 = re.indexOf("30 random results from") + "30 random results from".length();
	    	    int i2 = i1 + re.substring(i1).indexOf("found");
	    	    
	    	    if (i1>-1 && i2>-1 && i1<i2) result[k] = Integer.valueOf(re.substring(i1,i2).trim());
	    	    else result[k] = 0;
	
	    	    webClient.closeAllWindows();

			} catch (FailingHttpStatusCodeException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			} catch (MalformedURLException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			} catch (IOException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}

       	}
			return result;

    			        	
    		/**
    			            HttpClient httpclient = new DefaultHttpClient();
    			            try {
    			                
    			                System.out.println("Initial set of cookies:");
    			                List<Cookie> cookies = ((AbstractHttpClient) httpclient).getCookieStore().getCookies();
    			                if (cookies.isEmpty()) {
//    			                    System.out.println("None");
    			                } else {
    			                    for (int i = 0; i < cookies.size(); i++) {
//   			                        System.out.println("- " + cookies.get(i).toString());
    			                    }
    			                }
    			                
    			               List<NameValuePair> formparams = new ArrayList<NameValuePair>();

    			               System.out.println("SEARCH FOR " + text);
    			               formparams.add(new BasicNameValuePair("query", text));
    			               formparams.add(new BasicNameValuePair("submit", "submit"));
    			               
    			                               
    			                UrlEncodedFormEntity entity = new UrlEncodedFormEntity(formparams, "UTF-8");
    			                HttpPost httppost = new HttpPost("http://search.dcl.bas.bg/");
    			                httppost.setEntity(entity);

    			                
    			                HttpResponse response1 = httpclient.execute(httppost);

    			                System.out.println(response1.getStatusLine());
    			                
    			               
    			               HeaderElementIterator it = new BasicHeaderElementIterator(response1.headerIterator());

    			            		while (it.hasNext()) {
    			            		    HeaderElement elem = it.nextElement(); 
    			            		    System.out.println(elem.getName() + " = " + elem.getValue());
    			            		    NameValuePair[] params = elem.getParameters();
    			            		    for (int i = 0; i < params.length; i++) {
    			            		        System.out.println(" " + params[i]);
    			            		    }
    			            		}
    			            		
    			            		HttpEntity myEntity = response1.getEntity();
    			            		System.out.println(myEntity.getContentType());
    			            		System.out.println(myEntity.getContentLength());
    			            		System.out.println(EntityUtils.getContentCharSet(myEntity));
//    			            		System.out.println(EntityUtils.toString(myEntity));
    			                 
    			                System.out.println("Post logon cookies:");
    			                cookies = ((AbstractHttpClient) httpclient).getCookieStore().getCookies();
    			                if (cookies.isEmpty()) {
    			                    System.out.println("None");
    			                } else {
    			                    for (int i = 0; i < cookies.size(); i++) {
    			                        System.out.println("- " + cookies.get(i).toString());
    			                    }
    			                }
    			                
    			                String res = EntityUtils.toString(myEntity);
    			        		
    			        		InputStream is = new ByteArrayInputStream(res.getBytes());
    			        		 
    			        		// read it with BufferedReader
    			        		BufferedReader br = new BufferedReader(new InputStreamReader(is));

    			                String line;
    		readline:   		while ((line = br.readLine()) != null) {
    									System.out.println(line);
    			                }
    			           		
    			                httppost.abort();
    		} catch (IOException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			} finally{
				
			}
			*/
    		
    		
 
 
	}



}
